@@ -1,11 +1,12 @@
 """beatbox: Makes the salesforce.com SOAP API easily accessible."""
 
-__version__ = '0.12'
-__author__ = "Simon Fell"
+__version__ = '16.0'
+__author__ = "Simon Fell et al"
 __credits__ = "Mad shouts to the sforce possie"
 __copyright__ = "(C) 2006 Simon Fell. GNU GPL 2."
 
 import httplib
+import logging
 from urlparse import urlparse
 from StringIO import StringIO
 import gzip
@@ -22,7 +23,7 @@ _sobjectNs = "urn:sobject.partner.soap.sforce.com"
 _envNs = "http://schemas.xmlsoap.org/soap/envelope/"
 _noAttrs = AttributesNSImpl({}, {})
 
-DEFAULT_SERVER_URL = 'https://www.salesforce.com/services/Soap/u/7.0'
+DEFAULT_SERVER_URL = 'https://www.salesforce.com/services/Soap/u/16.0'
 
 # global constants for xmltramp namespaces, used to access response data
 _tPartnerNS = xmltramp.Namespace(_partnerNs)
@@ -34,6 +35,7 @@ gzipRequest=True    # are we going to gzip the request ?
 gzipResponse=True   # are we going to tell teh server to gzip the response ?
 forceHttp=False     # force all connections to be HTTP, for debugging
 
+logger = logging.getLogger('beatbox')
 
 def makeConnection(scheme, host):
     if forceHttp or scheme.upper() == 'HTTP':
@@ -49,7 +51,7 @@ class Client:
         self.__conn = None
 
     def __del__(self):
-        if self.__conn != None:
+        if callable(getattr(self.__conn, 'close', None)):
             self.__conn.close()
 
     # login, the serverUrl and sessionId are automatically handled, returns the loginResult structure       
@@ -72,6 +74,9 @@ class Client:
     def queryMore(self, queryLocator):
         return QueryMoreRequest(self.__serverUrl, self.sessionId, self.batchSize, queryLocator).post(self.__conn)
         
+    def search(self, sosl):
+        return SearchRequest(self.__serverUrl, self.sessionId, self.batchSize, sosl).post(self.__conn)
+        
     def getUpdated(self, sObjectType, start, end):
         return GetUpdatedRequest(self.__serverUrl, self.sessionId, sObjectType, start, end).post(self.__conn)
         
@@ -194,9 +199,7 @@ class XmlWriter:
         elif isinstance(s, datetime.date):
             # todo, try isoformat
             s = "%04d-%02d-%02d" % (s.year, s.month, s.day)
-        elif isinstance(s, int):
-            s = str(s)
-        elif isinstance(s, float):
+        elif isinstance(s, (int, float, long)):
             s = str(s)
         self.xg.characters(s)
 
@@ -284,13 +287,14 @@ class SoapEnvelope:
     #   todo: check for mU='1' headers
     #   returns the relevant result from the body child
     def post(self, conn=None, alwaysReturnList=False):
+        logger.debug(self.__class__)
         headers = { "User-Agent": "BeatBox/" + __version__,
                     "SOAPAction": "\"\"",
                     "Content-Type": "text/xml; charset=utf-8" }
         if gzipResponse:
             headers['accept-encoding'] = 'gzip'
         if gzipRequest:
-            headers['content-encoding'] = 'gzip'                    
+            headers['content-encoding'] = 'gzip'
         close = False
         (scheme, host, path, params, query, frag) = urlparse(self.serverUrl)
         max_attempts = 3
@@ -304,14 +308,14 @@ class SoapEnvelope:
                 conn.request("POST", path, self.makeEnvelope(), headers)
                 response = conn.getresponse()
                 rawResponse = response.read()
-            except:
+            except httplib.HTTPException:
                 if conn != None:
                     conn.close()
                     conn = None
                     response = None
                 attempt += 1
         if not response:
-            raise 'No response from Salesforce'
+            raise RuntimeError, 'No response from Salesforce'
         
         if response.getheader('content-encoding','') == 'gzip':
             rawResponse = gzip.GzipFile(fileobj=StringIO(rawResponse)).read()
@@ -400,7 +404,16 @@ class QueryMoreRequest(QueryOptionsRequest):
         
     def writeBody(self, s):
         s.writeStringElement(_partnerNs, "queryLocator", self.__queryLocator)
+
+
+class SearchRequest(QueryOptionsRequest):
+    def __init__(self, serverUrl, sessionId, batchSize, sosl):
+        QueryOptionsRequest.__init__(self, serverUrl, sessionId, batchSize, "search")
+        self.__search = sosl
         
+    def writeBody(self, s):
+        s.writeStringElement(_partnerNs, "searchString", self.__search)
+
 
 class GetUpdatedRequest(AuthenticatedRequest):
     def __init__(self, serverUrl, sessionId, sObjectType, start, end, operationName="getUpdated"):
